﻿///////////////////////////////////////////////////////////////////////////////
//
// (C) 2016 ICE TEA GROUP LLC - ALL RIGHTS RESERVED
//
// 
//
// ALL INFORMATION CONTAINED HEREIN IS, AND REMAINS
// THE PROPERTY OF ICE TEA GROUP LLC AND ITS SUPPLIERS, IF ANY.
// THE INTELLECTUAL PROPERTY AND TECHNICAL CONCEPTS CONTAINED
// HEREIN ARE PROPRIETARY TO ICE TEA GROUP LLC AND ITS SUPPLIERS
// AND MAY BE COVERED BY U.S. AND FOREIGN PATENTS, PATENT IN PROCESS, AND
// ARE PROTECTED BY TRADE SECRET OR COPYRIGHT LAW.
//
// DISSEMINATION OF THIS INFORMATION OR REPRODUCTION OF THIS MATERIAL
// IS STRICTLY FORBIDDEN UNLESS PRIOR WRITTEN PERMISSION IS OBTAINED
// FROM ICE TEA GROUP LLC.
//
///////////////////////////////////////////////////////////////////////////////


using System;
using System.Linq;
using System.ComponentModel;
using System.Diagnostics;
using System.Web;
using System.Web.UI.WebControls;
using Wisej.Core;
using Telerik.Web.UI;

namespace Wisej.Web.Ext.UpdatePanelAspNetControl
{
	/// <summary>
	/// Base non-generic class for <see T:"Wisej.Web.Ext.AspNetControl.AspNetWrapperBase"/>.
	/// </summary>
	[ToolboxItem(false)]
	public abstract class UpdatePanelAspNetWrapperBase : IFramePanel
	{
		// used to change the url of the inner panel when calling Update. 
		private int _version;

		/// <summary>
		/// Initializes a new instance of <see cref="T:Wisej.Web.Ext.UpdatePanelAspNetControl.UpdatePanelAspNetWrapper"/>.
		/// </summary>
		public UpdatePanelAspNetWrapperBase()
		{
			this.UseSessionViewState = true;
		}

		/// <summary>
		/// Fired when the wrapped control is initialized. Corresponds to the Page.PreInit event in the
		/// ASP.NET page life cycle.
		/// </summary>
		public event EventHandler PreInit;

		/// <summary>
		/// Fired when the wrapped control is initialized. Corresponds to the Page.Init event in the
		/// ASP.NET page life cycle.
		/// </summary>
		public event EventHandler Init;

		/// <summary>
		/// Fired when the wrapped control is loaded. Corresponds to the Page.Load event in the
		/// ASP.NET page life cycle.
		/// </summary>
		public event EventHandler Load;

		/// <summary>
		/// Fires the Init event when the wrapped control is initialized.
		/// </summary>
		/// <param name="e"></param>
		protected virtual void OnPreInit(EventArgs e)
		{
			this.PreInit?.Invoke(this, e);
		}

		/// <summary>
		/// Fires the Init event when the wrapped control is initialized.
		/// </summary>
		/// <param name="e"></param>
		protected virtual void OnInit(EventArgs e)
		{
			this.Init?.Invoke(this, e);
		}

		/// <summary>
		/// Fires the Load event when the wrapped control is initialized.
		/// </summary>
		/// <param name="e"></param>
		protected virtual void OnLoad(EventArgs e)
		{
			this.Load?.Invoke(this, e);
		}

		/// <summary>
		/// Determines whether Wisej saves the VIEWSTATE in memory. The default is true.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public bool UseSessionViewState
		{
			get;
			set;
		}

		/// <summary>
		/// Returns the instance of the wrapped ASP.NET control.
		/// It is valid only during the processing of the Init or the Load events.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public WebControl WrappedControl
		{
			get { return this._hostedControl; }
		}
		private WebControl _hostedControl;

        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public System.Web.UI.UpdatePanel UpdatePanel
        {
            get { return this._UpdatePanel; }
        }
        private System.Web.UI.UpdatePanel _UpdatePanel;

        /// <summary>
        /// Returns whether the current request is a postback.
        /// </summary>
        [Browsable(false)]
		public bool IsPostBack
		{
			get;
			private set;
		}

		/// <summary>
		/// Returns whether the current request is a callback.
		/// </summary>
		[Browsable(false)]
		public bool IsCallback
		{
			get;
			private set;
		}

		/// <summary>
		/// Returns whether the current request is an asynchronous postback.
		/// </summary>
		[Browsable(false)]
		public bool IsAsync
		{
			get;
			private set;
		}

		/// <summary>
		/// Returns the instance of the internal page used to wrap the ASP.NET control.
		/// It is valid only during the processing of the Init or the Load events.
		/// </summary>
		[Browsable(false)]
		public System.Web.UI.Page Page
		{
			get { return this._page; }
		}
		private System.Web.UI.Page _page;

		/// <summary>
		/// Returns source Url of the IFrame.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Url
		{
			get
			{
				// when in design mode, feed the design content from this control.
				if (((IWisejControl)this).DesignMode)
				{
					return "";
				}

				// use the embedded AspNetHost page passing the id to this instance of the
				// Wisej wrapper control. the page code will use the id to callback this control
				// during the page life cycle.
				return "Wisej.UpdatePanelAspNetHost.aspx?_cid=" + ((IWisejComponent)this).Id + "&_sc=" + this._version;
			}

			set { }
		}

		/// <summary>
		/// VIEWSTATE saved using Wisej sessions.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public object ViewState
		{
			get;
			set;
		}

		/// <summary>
		/// Returns the instance of the <see cref="T:System.Web.UI.ScriptManager"/> in the AspNetHost.asp page
		/// used to wrap AspNet controls.
		/// </summary>
		public System.Web.UI.ScriptManager ScriptManager
		{
			get
			{
				return System.Web.UI.ScriptManager.GetCurrent(this.Page);
			}
		}

		// Page.PreInit callback. Call comes from Wisej.AspNetHost.aspx.
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void OnPagePreInitCallback(System.Web.UI.Page page)
		{
			System.Diagnostics.Trace.TraceInformation("UpdatePanelAspNetWrapperBase.OnInitCallback");

			Debug.Assert(page != null);

			this.IsPostBack = page.IsPostBack;
			this.IsCallback = page.IsCallback;
			this.IsAsync = page.IsAsync;

			this._page = page;
			this._hostedControl = CreateHostedControl();

            this._UpdatePanel = new System.Web.UI.UpdatePanel();
            this._UpdatePanel.ID = ((IWisejControl)this).Id + "_" + typeof(System.Web.UI.UpdatePanel).Name;
            this._UpdatePanel.UpdateMode = System.Web.UI.UpdatePanelUpdateMode.Conditional;

            try
			{
				OnPreInit(EventArgs.Empty);
			}
			finally
			{
				this._page = null;
				// clear _hostedControl it after Load.
			}
		}

		// Page.Init callback. Call comes from Wisej.AspNetHost.aspx.
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void OnPageInitCallback(System.Web.UI.Page page)
		{
			System.Diagnostics.Trace.TraceInformation("UpdatePanelAspNetWrapperBase.OnInitCallback");

			Debug.Assert(page != null);
			Debug.Assert(this._hostedControl != null);

			this._page = page;
			try
			{
                if (this.WrappedControl != null)
                {
                    page.Form.Controls.Add(this.UpdatePanel);
                    this.UpdatePanel.ContentTemplateContainer.Controls.Add(this.WrappedControl);
                }

				OnInit(EventArgs.Empty);
			}
			finally
			{
				this._page = null;
				// clear _hostedControl it after Load.
			}
		}

		// Page.Load callback. Call comes from Wisej.AspNetHost.aspx.
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void OnPageLoadCallback(System.Web.UI.Page page)
		{
			System.Diagnostics.Trace.TraceInformation("UpdatePanelAspNetWrapperBase.OnLoadCallback");

			Debug.Assert(page != null);
			Debug.Assert(this._hostedControl != null);

			this._page = page;
			try
			{
				OnLoad(EventArgs.Empty);
			}
			finally
			{
				this._page = null;
				this._hostedControl = null;
			}
		}

		/// <summary>
		/// Causes the control to update the corresponding client side widget.
		/// When in design mode, causes the rendered control to update its
		/// entire surface in the designer.
		/// </summary>
		public new void Update()
		{
			this._version++;
			if (this._version == int.MaxValue)
				this._version = 0;

			base.Update();
		}

		/// <summary>
		/// Restores the Wisej session on the current thread from the specified
		/// HttpContext instance.
		/// </summary>
		/// <param name="context">HttpContext with the id of the session to restore.</param>
		public static void RestoreSession(HttpContext context)
		{
		}

		/// <summary>
		/// Creates the wrapped control.
		/// </summary>
		/// <returns></returns>
		protected abstract WebControl CreateHostedControl();
	}
}
